package data

import (
	"context"
	"eta/eta_task/services/alarm_msg"
	"eta/eta_task/utils"
	"fmt"
)

// RefreshPCSGBloombergDaily 中石油新加坡-刷新彭博日度指标
func RefreshPCSGBloombergDaily(cont context.Context) (err error) {
	defer func() {
		if err != nil {
			tips := "RefreshPCSGBloombergDaily-中石油新加坡-刷新彭博日度指标失败, ErrMsg:\n" + err.Error()
			utils.FileLog.Info(tips)
			go alarm_msg.SendAlarmMsg(tips, 3)
		}
	}()

	param := make(map[string]interface{})
	uri := "bloomberg/pcsg/refresh_daily"
	res, e := postRefreshEdbData(param, uri)
	if e != nil {
		err = fmt.Errorf("postRefreshEdbData err: %s", e.Error())
		return
	}
	if res != nil && res.Ret != 200 {
		err = fmt.Errorf("postRefreshEdbData fail")
		return
	}
	return
}

// RefreshPCSGBloombergWeekly 中石油新加坡-刷新彭博周度指标
func RefreshPCSGBloombergWeekly(cont context.Context) (err error) {
	defer func() {
		if err != nil {
			tips := "RefreshPCSGBloombergWeekly-中石油新加坡-刷新彭博周度指标失败, ErrMsg:\n" + err.Error()
			utils.FileLog.Info(tips)
			go alarm_msg.SendAlarmMsg(tips, 3)
		}
	}()

	param := make(map[string]interface{})
	uri := "bloomberg/pcsg/refresh_weekly"
	res, e := postRefreshEdbData(param, uri)
	if e != nil {
		err = fmt.Errorf("postRefreshEdbData err: %s", e.Error())
		return
	}
	if res != nil && res.Ret != 200 {
		err = fmt.Errorf("postRefreshEdbData fail")
		return
	}
	return
}

// RefreshPCSGBloombergMonthly 中石油新加坡-刷新彭博月度指标
func RefreshPCSGBloombergMonthly(cont context.Context) (err error) {
	defer func() {
		if err != nil {
			tips := "RefreshPCSGBloombergMonthly-中石油新加坡-刷新彭博月度指标失败, ErrMsg:\n" + err.Error()
			utils.FileLog.Info(tips)
			go alarm_msg.SendAlarmMsg(tips, 3)
		}
	}()

	param := make(map[string]interface{})
	uri := "bloomberg/pcsg/refresh_monthly"
	res, e := postRefreshEdbData(param, uri)
	if e != nil {
		err = fmt.Errorf("postRefreshEdbData err: %s", e.Error())
		return
	}
	if res != nil && res.Ret != 200 {
		err = fmt.Errorf("postRefreshEdbData fail")
		return
	}
	return
}
