package utils

import (
	"fmt"
	beego "github.com/beego/beego/v2/adapter"
	"github.com/beego/beego/v2/server/web"
	"strconv"
)

var (
	RunMode        string //运行模式
	MYSQL_URL      string //数据库连接
	MYSQL_URL_RDDP string //数据库连接
	MYSQL_URL_EDB  string
	MYSQL_URL_DATA string
	MYSQL_URL_GL   string
	MYSQL_URL_ETA  string

	REDIS_CACHE string //缓存地址
	//Rc          *cache.Cache //redis缓存
	Re error       //redis错误
	Rc RedisClient //redis缓存
)

var (
	LogPath    string
	LogFile    string
	BinLogPath string
	BinLogFile string
	LogMaxDays int //日志最大保留天数
)

var (
	APPNAME                        string //项目中文名称
	EmailSendToUsers               string // 邮件提醒人员
	RefreshEdbInfoEmailSendToUsers string // 刷新完成通知邮件人员
	// AlarmMsgUrl 报警服务地址
	AlarmMsgUrl string
)

// 经济数据库
var (
	EDB_LIB_URL         string
	APP_EDB_LIB_NAME_EN string
	EDB_LIB_Md5_KEY     string
)

// 公共数据服务接口参数
var (
	APPID  string
	SECRET string
)

var (
	WxAppId                     string
	WxAppSecret                 string
	TemplateId                  string
	RemindTemplateId            string
	AppointmentRemindTemplateId string //活动预约通知模板id
	YbCommonTemplateId          string //研报小程序消息模板ID
	SendWxTemplateEnable        string //是否开启模版消息，1是发送，其他不发送

	WxYbAppId string //微信研报小程序

	//内部员工公众号（弘则部门）
	AdminWxAppId     string
	AdminWxAppSecret string
)

// 模板消息推送
var (
	SendWxTemplateMsgUrl string
)

// BusinessCode 商家编码

var (
	BusinessCode  string //商户号
	WindServerUrl string //wind api 接口服务器地址
	LtServerUrl   string //路透 api 接口服务器地址
	PbServerUrl   string //彭博 api 接口服务器地址
)

// ES配置
var (
	ES_URL      string // ES服务器地址
	ES_USERNAME string // ES账号
	ES_PASSWORD string // ES密码
)

// ES索引配置
var (
	EsReportIndexName        string //研报ES索引
	EsEnglishReportIndexName string //英文研报ES索引
	SmartReportIndexName     string //智能研报ES索引
)

// 科大讯飞--语音合成
var (
	XfHostUrl string
)

var (
	TemplateIdByProduct string //产品运行报告通知-模板ID
)

// 阿里云配置
var (
	Bucketname       string
	Endpoint         string
	Imghost          string
	UploadDir        string
	Upload_Audio_Dir string
	AccessKeyId      string
	AccessKeySecret  string
)

var (
	// 是否xy
	IsXy   string
	IsPCSG string
)

var (
	EtaBridgeUrl          string // 桥接服务地址
	EtaBridgeAppNameEn    string // 桥接服务英文名称-鉴权用
	EtaBridgeMd5Key       string // 桥接服务Md5密钥-鉴权用
	EtaBridgeDesKey       string // 桥接服务Des密钥-解密数据用
	SyncUserPath          string // 桥接服务-用户同步服务地址
	SyncUserTime          string // 桥接服务-用户同步的时间配置
	SyncIndexPath         string // 桥接服务-指标同步服务地址
	SyncIndexTime         string // 桥接服务-指标同步的时间配置
	SyncIndexClassifyTime string // 桥接服务-指标分类同步的时间配置
	SyncCrmIndexPath      string // 桥接服务-crm指标同步服务地址
	SyncCrmIndexTime      string // 桥接服务-crm指标同步的时间配置
	SyncCrmAssetPkgCd     string // 桥接服务-crm资产包配置
	SyncCrmDataSourceType string // 桥接服务-crm内部来源系统参数配置
	SyncCrmIndexNum       int    // 桥接服务-每次同步crm数据的数量
)

var (
	EtaHubUrl string // 公共服务地址
)

var (
	PbRefreshFrequency string // 彭博更新频度
	PbRefreshOpen      string // 彭博刷新开关 0关闭，默认都是开启的
)
var HzDataApi string

func init() {
	tmpRunMode, err := web.AppConfig.String("run_mode")
	if err != nil {
		panic("配置文件读取run_mode错误 " + err.Error())
	}
	RunMode = tmpRunMode
	if RunMode == "" {
		localIp, err := GetLocalIP()
		fmt.Println("localIp:", localIp)
		if localIp == "10.0.0.123" {
			RunMode = "debug"
		} else {
			RunMode = "release"
		}
		//RunMode = "release"
		configPath := `/home/code/config/eta_task/conf/app.conf`
		err = web.LoadAppConfig("ini", configPath)
		if err != nil {
			fmt.Println("web.LoadAppConfig Err:" + err.Error())
		}
	}

	config, err := web.AppConfig.GetSection(RunMode)
	if err != nil {
		panic("配置文件读取错误 " + err.Error())
	}
	beego.Info(RunMode + " 模式")
	MYSQL_URL = config["mysql_url"]
	MYSQL_URL_RDDP = config["mysql_url_rddp"]

	MYSQL_URL_EDB = config["mysql_url_edb"]
	MYSQL_URL_DATA = config["mysql_url_data"]
	MYSQL_URL_GL = config["mysql_url_gl"]
	MYSQL_URL_ETA = config["mysql_url_eta"]

	REDIS_CACHE = config["beego_cache"]
	if len(REDIS_CACHE) <= 0 {
		panic(any("redis链接参数没有配置"))
	}
	//Rc, Re = cache.NewCache(REDIS_CACHE) //初始化缓存
	//if Re != nil {
	//	fmt.Println(Re)
	//	panic(any(Re))
	//}
	// 初始化缓存
	redisClient, err := initRedis(config["redis_type"], config["beego_cache"])
	if err != nil {
		fmt.Println("redis链接异常：", err)
		panic(any(Re))
	}
	Rc = redisClient

	// 项目中文名称
	appNameCn, err := web.AppConfig.String("app_name_cn")
	if err != nil {
		panic(any("配置文件读取app_name_cn错误 " + err.Error()))
	}
	APPNAME = appNameCn

	{
		LogPath = config["log_path"]
		LogFile = config["log_file"]
		BinLogPath = config["binlog_path"]
		BinLogFile = config["binlog_file"]
		logMaxDaysStr := config["log_max_day"]
		LogMaxDays, _ = strconv.Atoi(logMaxDaysStr)
	}
	// 邮件提醒人员
	EmailSendToUsers = config["email_send_to_users"]

	// 刷新完成通知邮件人员
	RefreshEdbInfoEmailSendToUsers = config["refresh_edb_info_email_send_to_users"]

	// 报警服务地址
	AlarmMsgUrl = config["alarm_msg_url"]

	// 公共指标库相关
	{
		EDB_LIB_URL = config["edb_lib_url"]
		APP_EDB_LIB_NAME_EN = config["app_edb_lib_name_en"]
		EDB_LIB_Md5_KEY = config["edb_lib_md5_key"]
	}

	APPID = config["appid"]
	SECRET = config["secret"]

	// 商家编码
	BusinessCode = config["business_code"]
	//服务检测
	WindServerUrl = config["wind_server_url"]
	LtServerUrl = config["lt_server_url"]
	PbServerUrl = config["pb_server_url"]

	// ES配置
	{
		ES_URL = config["es_url"]
		ES_USERNAME = config["es_username"]
		ES_PASSWORD = config["es_password"]
	}

	// ES 索引
	{
		EsReportIndexName = config["es_report_index_name"]
		EsEnglishReportIndexName = config["es_english_report_index_name"]
		SmartReportIndexName = config["es_smart_report_index_name"]
	}

	// 科大讯飞
	{
		XfHostUrl = config["xf_host_url"]
	}

	// OSS相关
	{
		Endpoint = config["endpoint"]
		Bucketname = config["bucket_name"]
		Imghost = config["img_host"]
		UploadDir = config["upload_dir"]
		Upload_Audio_Dir = config["upload_audio_dir"]
		AccessKeyId = config["access_key_id"]
		AccessKeySecret = config["access_key_secret"]
	}

	// 微信模版消息
	{
		SendWxTemplateEnable = config["send_wx_template_enable"]
	}

	// 客户信息
	{
		IsXy = config["is_xy"]
		IsPCSG = config["is_pcsg"]
	}

	// 桥接服务
	{
		EtaBridgeUrl = config["eta_bridge_url"]                     // 桥接服务地址
		EtaBridgeAppNameEn = config["eta_bridge_app_name_en"]       // 桥接服务英文名称-鉴权用
		EtaBridgeMd5Key = config["eta_bridge_md5_key"]              // 桥接服务Md5密钥-鉴权用
		EtaBridgeDesKey = config["eta_bridge_des_key"]              // 桥接服务Des密钥-解密数据用
		SyncUserPath = config["sync_user_path"]                     // 桥接服务-用户同步服务地址
		SyncUserTime = config["sync_user_time"]                     // 桥接服务-用户同步的时间配置
		SyncIndexPath = config["sync_index_path"]                   // 桥接服务-指标同步服务地址
		SyncIndexTime = config["sync_index_time"]                   // 桥接服务-指标同步的时间配置
		SyncIndexClassifyTime = config["sync_index_classify_time"]  // 桥接服务-指标分类同步的时间配置
		SyncCrmIndexPath = config["sync_crm_index_path"]            // 桥接服务-crm指标同步服务地址
		SyncCrmIndexTime = config["sync_crm_index_time"]            // 桥接服务-crm指标同步的时间配置
		SyncCrmAssetPkgCd = config["sync_crm_asset_pkg_cd"]         // 桥接服务-crm资产包配置
		SyncCrmDataSourceType = config["sync_crm_data_source_type"] // 桥接服务-crm内部来源系统参数配置

		syncCrmIndexNumStr := config["sync_crm_index_num"] // 桥接服务-每次同步crm数据的数量
		if syncCrmIndexNumStr != `` {
			SyncCrmIndexNum, _ = strconv.Atoi(syncCrmIndexNumStr)
		}
		if SyncCrmIndexNum <= 0 {
			SyncCrmIndexNum = 2000
		}
	}

	// 公共服务
	{
		EtaHubUrl = config["eta_hub_url"] // 公共服务地址
	}

	HzDataApi = config["hz_data_api"]
	if HzDataApi == "" {
		HzDataApi = "https://hzdataapi.hzinsights.com/hzdataapi/"
	}

	// 彭博刷新频度
	PbRefreshFrequency = config["pb_refresh_frequency"]
	// 彭博刷新开关
	PbRefreshOpen = config["pb_refresh_open"]
}
